#ifndef _DTRUETYPEFONT_H
#define _DTRUETYPEFONT_H

// This file was automatically generated by objch.

//==============================================================================
//
//         DTrueTypeFont - the true type font class in the ofc-library
//
//               Copyright (C) 2007  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2007/11/04 15:29:14 $ $Revision: 1.4 $
//
//==============================================================================
#ifdef HAVE_DTRUETYPEFONT
#endif

#include "ofc/config.h"

#include "ofc/DFont.h"

#if defined(HAS_FREETYPE)
#define HAVE_DTRUETYPEFONT 1
#endif


#ifdef HAVE_DTRUETYPEFONT

//
// The DTrueTypeFont class implements methods for using a TrueType font. 
// Actually every font that can be loaded with the FreeType library, can 
// be used by this class.
// 

@interface DTrueTypeFont : Object <DFont>
{
@private
  void     *_font;    // the loaded font
}

#endif
#ifdef HAVE_DTRUETYPEFONT

//// Constructors

//
// Initialise an graphic drawable object
//
// @return the object
//
- (DTrueTypeFont *) init;


//// Copy related methods

//
// Do a shallow copy of the graphic drawable object
//
// @return the object
//
- shallowCopy;


//// Deconstructor

//
// Free the truetype font
// 
// @return the object
// 
- free;


//// Font members

//
// Get the family name of the font
//
// @return the family name (or NULL)
//
- (const char *) familyName;

//
// Get the style name of the font
//
// @return the style name (or NULL)
//
- (const char *) styleName;

//
// Get the number of characters in font
//
// @return the number of characters in the font (or -1)
//
- (int) glyphsInFont;

#if 0
#endif

//// Font methods

//
// Open a (true type) font file
//
// @param filename      the name of the font file
// 
// @return success
//
- (BOOL) open :(const char *) filename;

//
// Check if the font is open
//
// @return is it ?
//
- (BOOL) isOpen;

//
// Close the font (file)
//
// @return success
//
- (BOOL) close;

//
// Set the font size based on the width and height of one character
// 
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(unsigned) width :(unsigned) height;

//
// Set the font size based on a string and requested a size
// 
// @param cstr          the string
// @param width         the requested width
// @param height        the requested height
//
// @return success
//
- (BOOL) size :(const char *) cstr :(unsigned) width :(unsigned) height;

//
// Get the size of a string for the current font size
// 
// @param cstr          the string
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) stringSize :(const char *) cstr :(unsigned *) width :(unsigned *) height;

//
// Get the size of a character for the current font and size
// 
// @param ch            the character
// @param width         the width of the string
// @param height        the height of the string
//
// @return success
//
- (BOOL) charSize :(char) ch :(unsigned *) width :(unsigned *) height;

//
// Get the glyph info for blitting a character for the current font and size
// 
// @param ch            the character
// @param glyph         the character bitmap and blit info
//
// @return success
//
- (BOOL) glyph :(char) ch :(DFT_Glyph *) glyph;

@end
#endif
#endif
